﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Web;
using System.Xml;
using Dynamicweb.Ecommerce.LiveIntegration.Configuration;
using Dynamicweb.Ecommerce.Products;
using Dynamicweb.Security.UserManagement;
using Dynamicweb.Ecommerce.Orders;
using Dynamicweb.Ecommerce.International;
using Dynamicweb.Environment;
using Dynamicweb.Core;
using System.Globalization;

namespace Dynamicweb.Ecommerce.LiveIntegration
{
    internal class Helpers
    {
        public static string ProductIdentifier(Product product)
        {
            return ProductIdentifier(product.Id, product.VariantId, product.LanguageId);
        }

        public static string ProductIdentifier(string productID, string variantID, string languageID)
        {
            User user = User.GetCurrentUser();
            return string.Format("{0}.{1}.{2}.{3}", productID, variantID, languageID, (user != null) ? user.ExternalID : string.Empty);
        }

        public static string OrderIdentifier(Order order)
        {
            string ret = order.Id;
            foreach (OrderLine ol in order.OrderLines)
            {
                ret += string.Format(".{0}.{1}.{2}.{3}", ol.Id, ol.Quantity, ol.ProductId, ol.ProductVariantId);
            }
            return ret;
        }

        public static string CustomerIdentifier
        {
            get
            {
                return GetCustomerIdentifier(User.GetCurrentUser());
            }
        }

        public static string GetCustomerIdentifier(User user)
        {
            if (user != null)
            {
                if (Convert.ToBoolean(GetConfigValue(Constants.CustomFieldsSettings.Path, Constants.CustomFieldsSettings.AllowMultipleContactsPerCompany)))
                {
                    return user.CustomerNumber;
                }
                else
                {
                    return user.ExternalID;
                }
            }
            else
            {
                return Constants.ANONYMOUS_USER_NAME;
            }
        }

        public static void AddChildXmlNodeToCart(XmlDocument xDoc, XmlElement parent, string nodeName, string nodeValue)
        {
            XmlElement node = xDoc.CreateElement("column");
            node.SetAttribute("columnName", nodeName);
            node.InnerText = nodeValue;
            parent.AppendChild(node);
        }


        public static void AddChildXmlNodeToProductList(XmlDocument xDoc, XmlElement parent, string nodeName, string nodeValue)
        {
            XmlElement node = xDoc.CreateElement(nodeName);
            node.InnerText = nodeValue;
            parent.AppendChild(node);
        }
        /// <summary>
        /// Returns the relevant Enum Value. If input can be parsed to Enum type, then the enum value is returned. Otherwise defaultValue is returned.
        /// </summary>
        /// <typeparam name="T">Type of Enum to parse value from</typeparam>
        /// <param name="input">To be parsed as Enum value.</param>
        /// <param name="defaultValue">Default value if input was not valid enum value.</param>
        /// <returns></returns>
        public static T GetEnumValueFromString<T>(string input, T defaultValue)
        {
            if (string.IsNullOrEmpty(input))
                return defaultValue;

            T value;
            try
            {
                value = (T)Enum.Parse(typeof(T), input, true);
            }
            catch (Exception)
            {
                return defaultValue;
            }

            return value;
        }

        public static bool ParseResponseToXml(string response, out XmlDocument responseDoc)
        {
            bool result = true;
            responseDoc = new XmlDocument();
            try
            {
                responseDoc.LoadXml(response);
            }
            catch (Exception e)
            {
                result = false;
                Logger.Instance.Log(ErrorLevel.ResponseError, string.Format("Response is not valid XML format: '{0}'.", e.Message));
            }
            return result;
        }

        public static string GetConfigValue(string configPath, string key)
        {
            string result = null;
            try
            {
                var config = ConfigurationHandler.GetConfig(configPath);
                if (config != null && config.ContainsKey(key))
                    result = config[key];
            }
            catch (System.IO.FileNotFoundException)
            { }

            return result;
        }

        public static Currency CurrentCurrency
        {
            get
            {
                Currency ret = Ecommerce.Common.Context.Currency;
                if (!string.IsNullOrEmpty(Context.Current.Request.GetString("CurrencyCode")))
                {
                    ret = Ecommerce.Common.Application.get_Currency(Context.Current.Request.GetString("CurrencyCode"));
                }
                else if (!Common.Context.CurrencySetByCustomer && Dynamicweb.Frontend.PageView.Current() != null && Dynamicweb.Frontend.PageView.Current().Area != null &&
                    !string.IsNullOrEmpty(Convert.ToString(Dynamicweb.Frontend.PageView.Current().Area.EcomCurrencyId)))
                {
                    ret = Ecommerce.Common.Application.get_Currency(Convert.ToString(Dynamicweb.Frontend.PageView.Current().Area.EcomCurrencyId));
                }
                return ret;
            }
        }

        public static double ToDouble(string value)
        {
            double result;
            CultureInfo cultureInfo = GetCultureInfo();            
            if (cultureInfo != null)
            {
                try
                {
                    result = double.Parse(value, cultureInfo);
                }
                catch(FormatException ex)
                {
                    Logger.Instance.Log(ErrorLevel.Error, string.Format("Error parse '{0}' to double in '{1}' culture: '{2}'.",
                        value, cultureInfo.Name, ex.Message));
                    result = Converter.ToDouble(value);
                }                                
            }
            else
            {
                result = Converter.ToDouble(value);
            }
            return result;
        }

        private static CultureInfo GetCultureInfo()
        {
            CultureInfo result = null;
            if (Context.Current != null && Context.Current.Application != null && Context.Current.Application[Constants.WebServiceConfiguration.NumberFormatCulture] != null)
            {
                result = (CultureInfo)Context.Current.Application[Constants.WebServiceConfiguration.NumberFormatCulture];
            }
            else
            {
                string culture = GetConfigValue(Constants.WebServiceConfiguration.Path, Constants.WebServiceConfiguration.NumberFormatCulture);
                if (!string.IsNullOrEmpty(culture))
                {
                    try
                    {
                        result = CultureInfo.GetCultureInfo(culture);
                    }
                    catch (Exception ex)
                    {
                        Logger.Instance.Log(ErrorLevel.Error, string.Format("Error getting culture info for the culture: '{0}' message: '{1}'.", 
                            culture ,ex.Message));
                        result = null;
                    }
                }
                if (Context.Current != null && Context.Current.Application != null)
                {
                    Context.Current.Application[Constants.WebServiceConfiguration.NumberFormatCulture] = result;
                }
            }
            return result;
        }

        internal static void ClearCachedCultureInfo()
        {
            if (Context.Current != null && Context.Current.Application != null && Context.Current.Application[Constants.WebServiceConfiguration.NumberFormatCulture] != null)
            {
                Context.Current.Application.Remove(Constants.WebServiceConfiguration.NumberFormatCulture);
            }
        }
    }
}
